@@ -104,10 +104,8 @@ Command options:
     --timeformat=unix|human|iso8601
                              unix timestamp or human-readable? [default: human]
     --header=name|rid        include a header row
-    --chunkhours=<hours>     break read into multiple requests of length
-                             <hours>, printing data as it is received and
-                             ignoring --limit. Note that this requires start
-                             and end time be set.
+    --chunksize=<size>       [default: 212] break read into requests of
+                             length <size>, printing data as it is received.
     {{ helpoption }}
 
     If <rid> is omitted, reads all datasources and datarules under <cik>.
@@ -342,26 +340,31 @@ plugins = []
 if platform.system() != 'Windows':
     # load plugins. use timezone because this file may be running
     # as a script in some other location.
-    plugin_path = os.path.join(os.path.dirname(timezone.__file__), 'plugins')
+    default_plugin_path = os.path.join(os.path.dirname(timezone.__file__), 'plugins')
 
-    plugin_names = [os.path.basename(f)[:-3]
-        for f in glob.glob(plugin_path + "/*.py")
-        if not os.path.basename(f).startswith('_')]
+    plugin_paths = os.getenv('EXO_PLUGIN_PATH', default_plugin_path).split(':')
 
-    for module_name in plugin_names:
-        try:
-            plugin = importlib.import_module('plugins.' + module_name)
-        except:
-            plugin = importlib.import_module('exoline.plugins.' + module_name, package='test')
+    for plugin_path in [i for i in plugin_paths if len(i) > 0]:
+        plugin_names = [os.path.basename(f)[:-3]
+            for f in glob.glob(plugin_path + "/*.py")
+            if not os.path.basename(f).startswith('_')]
 
-        # instantiate plugin
-        p = plugin.Plugin()
-        plugins.append(p)
+        for module_name in plugin_names:
+            try:
+                plugin = importlib.import_module('plugins.' + module_name)
+            except:
+                plugin = importlib.import_module('exoline.plugins.' + module_name, package='test')
 
-        # get documentation
-        cmd_doc[p.command()] = plugin.__doc__
+            # instantiate plugin
+            p = plugin.Plugin()
+            plugins.append(p)
+
+            # get documentation
+            cmd_doc[p.command()] = plugin.__doc__
 else:
+    # plugin support for Windows executable build
     try:
+        # spec plugin
         try:
             from ..exoline.plugins import spec
         except:
@@ -369,6 +372,16 @@ else:
         p = spec.Plugin()
         plugins.append(p)
         cmd_doc[p.command()] = spec.__doc__
+
+        # transform plugin
+        try:
+            from ..exoline.plugins import transform
+        except:
+            from exoline.plugins import transform
+        p = transform.Plugin()
+        plugins.append(p)
+        cmd_doc[p.command()] = transform.__doc__
+
     except Exception as ex:
         import traceback
         traceback.print_exc()
@@ -579,7 +592,7 @@ class ExoRPC():
                  starttime=None,
                  endtime=None,
                  selection='all',
-                 chunkhours=None):
+                 chunksize=212):
         '''Generates multiple rids and returns combined timestamped data like this:
                [12314, [1, 77, 'a']
                [12315, [2, 78, None]]
@@ -591,57 +604,53 @@ class ExoRPC():
             responses = self._exomult(cik, [['read', rid, options] for rid in rids])
             return self._combinereads(responses, options['sort'])
 
-        def _read_chunk(cik, rids, options, start, end):
-            chunkoptions = options.copy()
-            chunkoptions['starttime'] = start
-            chunkoptions['endtime'] = end
-            # read all points
-            chunkoptions['limit'] = end - start
-            return _read(cik, rids, chunkoptions)
-
-        if chunkhours is None:
+        if limit <= chunksize :
             for r in _read(cik, rids, options):
                 yield r
         else:
-
-            # maximum # seconds we want to read in one request
-            # TODO: is there a clever way to calculate optimal read
-            # sizes automatially? What users want is 1.) no
-            # timeout/onep error and 2.) progress indication of some kind.
-            # I'd rather the user not have to figure out chunkhours
-            # themselves.
-            max_sec = 60 * 60 * int(chunkhours)
-
-            # TODO: figure out whether it's probably a large read by reading
-            # info, usage, or by some calculation.
-            is_large_read = True
-
-            # TODO: figure out earliest timestamp with data and adjust
-            # starttime if necessary.
-
-            if is_large_read:
-
-                if 'sort' in options and options['sort'] == 'desc':
-                    # descending
-                    for end in range(options['endtime'],
-                                     options['starttime'],
-                                     -max_sec):
-                        start = max(end - max_sec, options['starttime'])
-                        for r in _read_chunk(cik, rids, options, start, end):
-                            yield r
+            # Read chunks by limit.
+            maxLimit = options['limit']
+            if 'sort' in options and options['sort'] == 'desc':
+                # descending
+                if 'endtime' in options:
+                    nextStart = options['endtime']
                 else:
-                    # ascending
-                    for start in range(options['starttime'],
-                                       options['endtime'],
-                                       max_sec):
-                        end = min(start + max_sec, options['endtime'])
-                        for r in _read_chunk(cik, rids, options, start, end):
-                            yield r
-
+                    nextStart = ExoUtilities.parse_ts_tuple(datetime.now().timetuple())
+                while True:
+                    chunkOpt = options.copy()
+                    chunkOpt['endtime'] = nextStart
+                    chunkOpt['limit'] = chunksize
+                    res = _read(cik, rids, chunkOpt);
+                    if len(res) == 0:
+                        break
+                    maxLimit = maxLimit - len(res)
+                    if maxLimit <= 0:
+                        break;
+                    #save oldest
+                    nextStart = res[-1][0] - 1
+                    for r in res:
+                        yield r
             else:
-                # make a single request
-                for r in _read(cik, rids, options):
-                    yield r
+                # ascending
+                if 'starttime' in options:
+                    nextStart = options['starttime']
+                else:
+                    nextStart = 0
+                while True:
+                    chunkOpt = options.copy()
+                    chunkOpt['starttime'] = nextStart
+                    chunkOpt['limit'] = chunksize
+                    res = _read(cik, rids, chunkOpt);
+                    if len(res) == 0:
+                        break
+                    maxLimit = maxLimit - len(res)
+                    if maxLimit <= 0:
+                        break
+                    #save oldest
+                    nextStart = res[-1][0] + 1
+                    for r in res:
+                        yield r
+
 
     def write(self, cik, rid, value):
         isok, response = self.exo.write(cik, rid, value)
@@ -1537,44 +1546,49 @@ class ExoPortals():
             raise ExoException('Bad status from Portals cache invalidate API call: ' + ex)
 
 
-def parse_ts(s):
-    return None if s is None else parse_ts_tuple(parser.parse(s).timetuple())
+class ExoUtilities():
 
-def parse_ts_tuple(t):
-    return int(time.mktime(t))
+    @classmethod
+    def parse_ts(cls, s):
+        return None if s is None else ExoUtilities.parse_ts_tuple(parser.parse(s).timetuple())
 
-def is_ts(s):
-    return s is not None and re.match('^[0-9]+$', s) is not None
+    @classmethod
+    def parse_ts_tuple(cls, t):
+        return int(time.mktime(t))
 
-def get_startend(args):
-    '''Get start and end timestamps based on standard arguments'''
-    start = args.get('--start', None)
-    end = args.get('--end', None)
-    if is_ts(start):
-        start = int(start)
-    else:
-        start = parse_ts(start)
-    if end == 'now':
-        end = None
-    elif is_ts(end):
-        end = int(end)
-    else:
-        end = parse_ts(end)
-    return start, end
+    @classmethod
+    def get_startend(cls, args):
+        '''Get start and end timestamps based on standard arguments'''
+        start = args.get('--start', None)
+        end = args.get('--end', None)
+        def is_ts(s):
+            return s is not None and re.match('^[0-9]+$', s) is not None
+        if is_ts(start):
+            start = int(start)
+        else:
+            start = ExoUtilities.parse_ts(start)
+        if end == 'now':
+            end = None
+        elif is_ts(end):
+            end = int(end)
+        else:
+            end = ExoUtilities.parse_ts(end)
+        return start, end
 
-def format_time(sec):
-    '''Formats a time interval for human consumption'''
-    intervals = [[60 * 60 * 24, 'd'],
-                    [60 * 60, 'h'],
-                    [60, 'm']]
-    text = ""
-    for s, label in intervals:
-        if sec >= s and sec // s > 0:
-            text = "{0} {1}{2}".format(text, sec // s, label)
-            sec -= s * (sec // s)
-    if sec > 0:
-        text += " {0}s".format(sec)
-    return text.strip()
+    @classmethod
+    def format_time(cls, sec):
+        '''Formats a time interval for human consumption'''
+        intervals = [[60 * 60 * 24, 'd'],
+                        [60 * 60, 'h'],
+                        [60, 'm']]
+        text = ""
+        for s, label in intervals:
+            if sec >= s and sec // s > 0:
+                text = "{0} {1}{2}".format(text, sec // s, label)
+                sec -= s * (sec // s)
+        if sec > 0:
+            text += " {0}s".format(sec)
+        return text.strip()
 
 
 def spark(numbers, empty_val=None):
@@ -1686,8 +1700,8 @@ def show_intervals(er, cik, rid, start, end, limit, numstd=None):
 
     print(spark(bins, empty_val=0))
 
-    min_label = format_time(min_t)
-    max_label = format_time(max_t)
+    min_label = ExoUtilities.format_time(min_t)
+    max_label = ExoUtilities.format_time(max_t)
     sys.stdout.write(min_label)
     sys.stdout.write(' ' * (num_bins - len(min_label) - len(max_label)))
     sys.stdout.write(max_label + '\n')
@@ -1715,7 +1729,7 @@ def read_cmd(er, cik, rids, args):
     limit = 1 if limit is None else int(limit)
 
     # time range
-    start, end = get_startend(args)
+    start, end = ExoUtilities.get_startend(args)
 
     timeformat = args['--timeformat']
     if headertype == 'name':
@@ -1818,17 +1832,14 @@ def read_cmd(er, cik, rids, args):
 
             time.sleep(sleep_seconds)
     else:
-        chunkhours = args['--chunkhours']
-        if chunkhours is not None and (start is None or end is None):
-            raise ExoException(
-                "--chunkhours requires --start and --end be set")
+        chunksize = int(args['--chunksize'])
         result = er.readmult(cik,
                              rids,
                              sort=args['--sort'],
                              starttime=start,
                              endtime=end,
                              limit=limit,
-                             chunkhours=chunkhours)
+                             chunksize=chunksize)
         for t, v in result:
             printline(t, v)
 
@@ -1922,7 +1933,7 @@ def handle_args(cmd, args):
                         if match is None:
                             try:
                                 t, v = tv.split(',')
-                                t = parse_ts(t)
+                                t = ExoUtilities.parse_ts(t)
                                 entries.append([t, v])
                             except Exception:
                                 sys.stderr.write(
@@ -2066,7 +2077,7 @@ def handle_args(cmd, args):
                 # output json
                 pr(json.dumps(info))
         elif cmd == 'flush':
-            start, end = get_startend(args)
+            start, end = ExoUtilities.get_startend(args)
             er.flush(cik, rids, newerthan=start, olderthan=end)
         elif cmd == 'usage':
             allmetrics = ['client',
@@ -2078,7 +2089,7 @@ def handle_args(cmd, args):
                         'sms',
                         'xmpp']
 
-            start, end = get_startend(args)
+            start, end = ExoUtilities.get_startend(args)
             er.usage(cik, rids[0], allmetrics, start, end)
         # special commands
         elif cmd == 'tree':
@@ -2091,8 +2102,8 @@ def handle_args(cmd, args):
                             create=args['--create'])
         elif cmd == 'spark':
             days = int(args['--days'])
-            end = parse_ts_tuple(datetime.now().timetuple())
-            start = parse_ts_tuple((datetime.now() - timedelta(days=days)).timetuple())
+            end = ExoUtilities.parse_ts_tuple(datetime.now().timetuple())
+            start = ExoUtilities.parse_ts_tuple((datetime.now() - timedelta(days=days)).timetuple())
             numstd = args['--stddev']
             numstd = int(numstd) if numstd is not None else None
             show_intervals(er, cik, rids[0], start, end, limit=1000000, numstd=numstd)
@@ -2234,7 +2245,13 @@ def handle_args(cmd, args):
             handled = False
             for plugin in plugins:
                 if cmd in plugin.command():
-                    options = {'cik': cik, 'rpc': er, 'exception': ExoException}
+                    options = {
+                            'cik': cik,
+                            'rids': rids,
+                            'rpc': er,
+                            'exception': ExoException,
+                            'utils': ExoUtilities
+                            }
                     try:
                         options['data'] = ed
                     except NameError:
